import json
from json import JSONDecodeError
from typing import Any, Union, List, Optional

from fastvalidate.rule_abstract import RuleAbstract


class ListRule(RuleAbstract):
    @classmethod
    def slug(cls) -> Union[str, List[str]]:
        return ['list', 'array']

    def message(self) -> str:
        return 'Field must be array'

    def __list_from_string(self, string: str) -> Optional[List]:
        try:
            string = json.loads(string)
            if isinstance(string, list): return string
            return None
        except JSONDecodeError:
            return None

    def validate(self, field: Any, value: Any) -> bool:
        if isinstance(value, list): return True
        if isinstance(value, str): return self.__list_from_string(value) is not None
        return False

    def process(self, field: Any, value: Any) -> Any:
        if isinstance(value, list): return value
        if isinstance(value, str): return self.__list_from_string(value)
        return None



from typing import Any, Union, List

from fastvalidate.rule_abstract import RuleAbstract


class RequiredRule(RuleAbstract):

    @classmethod
    def slug(cls) -> Union[str, List[str]]:
        return ['required', 'req']

    def validate(self, field: Any, value: Any) -> bool:
        return value is not None and value != ''

    def process(self, field: Any, value: Any) -> Any:
        return value

    def message(self) -> str:
        return 'Field is required'



import re
from typing import Any, Union, List, Tuple, Optional

from fastvalidate.rule_abstract import RuleAbstract, ComplexRuleAbstract


class PasswordRule(ComplexRuleAbstract):

    @classmethod
    def slug(cls) -> Union[str, List[str]]:
        return ['password']

    def validate_with_message(self, field: Any, value: Any) -> Tuple[bool, Optional[str]]:
        if not isinstance(value, str):
            return False, 'Field must be string'

        if len(value) < 8:
            return False, 'Field must be at least 8 characters'

        if re.search(r"\d", value) is None:
            return False, 'Field must have at least one numeric characters'

        if re.search(r"[A-Z]", value) is None:
            return False, 'Field must have at least one uppercase characters'

        if re.search(r"[a-z]", value) is None:
            return False, 'Field must have at least one lowercase characters'

        if re.search(r"[ !#$%&'()*+,-./[\\\]^_`{|}~" + r'"]', value) is None:
            return False, 'Field must have at least one symbol characters'

        return True, None

    def process(self, field: Any, value: Any) -> Any:
        return value



from typing import Any

from pydantic import BaseModel as PydanticBaseModel, root_validator, validator, ValidationError
from pydantic.error_wrappers import ErrorWrapper
from pydantic.fields import ModelField

from fastvalidate import Validator


class BaseModel(PydanticBaseModel):
    @validator('*')
    def custom_validation(cls, value, field: ModelField, values, config):
        rules = getattr(config, 'rules', {})
        field_name = field.name
        validator_ = Validator({**values, field_name: value}, {field_name: rules.get(field_name, [])})
        result = validator_.validate()
        if result.fails(): raise ValueError(result.errors()[field_name])
        return value



from typing import Dict, List, Union, Type

from fastvalidate.rule_abstract import RuleAbstract
from fastvalidate.rules.boolean import BoolRule
from fastvalidate.rules.choice import ChoiceRule
from fastvalidate.rules.comparison import GreaterThanRule, GreaterThanEqualRule, LessThanRule, LessThanEqualRule
from fastvalidate.rules.dict import DictRule
from fastvalidate.rules.email import EmailRule
from fastvalidate.rules.len import LenRule
from fastvalidate.rules.list import ListRule
from fastvalidate.rules.max import MaxRule
from fastvalidate.rules.min import MinRule
from fastvalidate.rules.numeric import NumericRule
from fastvalidate.rules.password import PasswordRule
from fastvalidate.rules.regex import RegexRule
from fastvalidate.rules.required import RequiredRule
from fastvalidate.rules.url import UrlRule

AVAILABLE_RULES : List[Type[RuleAbstract]] = [
    RequiredRule,
    BoolRule,
    GreaterThanRule,
    GreaterThanEqualRule,
    LessThanRule,
    LessThanEqualRule,
    DictRule,
    ListRule,
    EmailRule,
    ChoiceRule,
    LenRule,
    MinRule,
    MaxRule,
    NumericRule,
    PasswordRule,
    RegexRule,
    UrlRule
]


class ValidationResult:

    def __init__(self, errors: Dict[str, str], values: Dict[str, str]) -> None:
        self.__errors = errors
        self.__values = values

    def errors(self) -> Dict[str, str]:
        return self.__errors

    def values(self) -> Dict[str, str]:
        return self.__values

    def fails(self) -> bool:
        return len(self.__errors.items()) > 0

    def passes(self) -> bool:
        return len(self.__errors.items()) == 0


class Validator:

    def __init__(self, data: Dict, rules: Dict[str, Union[List[Union[str, RuleAbstract]], str]]) -> None:
        self.__data = data
        self.__available_rules: Dict[str, Type[RuleAbstract]] = self.__parse_available_rules()
        self.__rules: Dict[str, List[RuleAbstract]] = self.__parse_rules(rules)

    def __parse_rules(self, rules: Dict[str, Union[List[Union[str, RuleAbstract]], str]]) -> Dict[str, List[RuleAbstract]]:
        for field, rule_set in rules.items():
            if isinstance(rule_set, str): rule_set = rule_set.split("|")
            for index, rule in enumerate(rule_set):
                if isinstance(rule, str): rule_set[index] = self.__rule_from_string(rule)
            rules[field] = rule_set

        return rules

    def __rule_from_string(self, rule: str) -> RuleAbstract:
        slug, separator, args = rule.partition(':')
        args = args.split(",") if separator != '' else []
        rule_class : Type[RuleAbstract] = self.__available_rules.get(slug, None)
        if rule_class is None: raise Exception(f"Rule with signature of {slug} is not supported")
        return rule_class(*args)

    def validate(self) -> ValidationResult:
        errors = dict()
        values = dict()
        for field, value in self.__data.items():
            rules: List[RuleAbstract] = self.__rules.get(field, [])
            for rule in rules:
                if rule.validate(field, value):
                    value = rule.process(field, value)
                else:
                    errors[field] = rule.message()
                    break
            values[field] = value

        return ValidationResult(errors, values)

    def __parse_available_rules(self):
        rules = dict()
        for rule in AVAILABLE_RULES:
            slugs = rule.slug()
            slugs = slugs if isinstance(slugs, list) else [slugs]
            rules = dict(**rules, **{slug: rule for slug in slugs})

        return rules



import re
from typing import Any, Union, List

from fastvalidate.rule_abstract import RuleAbstract


class UrlRule(RuleAbstract):

    @classmethod
    def slug(cls) -> Union[str, List[str]]:
        return ['url']

    def validate(self, field: Any, value: Any) -> bool:
        pattern = r"https?:\/\/(www\.)?[-a-zA-Z0-9@:%._\+~#=]{1,256}\.[a-zA-Z0-9()]{1,6}\b([-a-zA-Z0-9()@:%_\+.~#?&//=]*)"
        return re.fullmatch(pattern, value) is not None

    def process(self, field: Any, value: Any) -> Any:
        return value

    def message(self) -> str:
        return 'Field must be url'



import re
from typing import Any, Union, List

from fastvalidate.rule_abstract import RuleAbstract


class RegexRule(RuleAbstract):

    def __init__(self, pattern: str) -> None:
        super().__init__()
        self.__pattern = pattern

    @classmethod
    def slug(cls) -> Union[str, List[str]]:
        return ['regex']

    def validate(self, field: Any, value: Any) -> bool:
        return isinstance(value, str) and re.fullmatch(self.__pattern, value) is not None

    def process(self, field: Any, value: Any) -> Any:
        return value

    def message(self) -> str:
        return 'Field is not valid'



from typing import Any, Union, List

from fastvalidate.rule_abstract import RuleAbstract


class NumericRule(RuleAbstract):

    @classmethod
    def slug(cls) -> Union[str, List[str]]:
        return ['numeric', 'number', 'int', 'integer']

    def validate(self, field: Any, value: Any) -> bool:
        if isinstance(value, int): return True
        if isinstance(value, str): return value.isnumeric()
        return False

    def process(self, field: Any, value: Any) -> Any:
        if isinstance(value, int): return value
        if isinstance(value, str): return int(value)

    def message(self) -> str:
        return 'Field must be numeric'



from typing import Any, Union, List, Tuple, Optional

from fastvalidate.rule_abstract import ComplexRuleAbstract


class MinRule(ComplexRuleAbstract):

    def __init__(self, threshold: int) -> None:
        super().__init__()
        self.__threshold = int(threshold)

    @classmethod
    def slug(cls) -> Union[str, List[str]]:
        return ['min']

    def validate_with_message(self, field: Any, value: Any) -> Tuple[bool, Optional[str]]:
        if isinstance(value, int):
            return value >= self.__threshold, f"Field must be greater that or equal {self.__threshold}"

        if isinstance(value, float):
            return value >= self.__threshold, f"Field must be greater that or equal {self.__threshold}"

        if isinstance(value, str):
            return len(value) >= self.__threshold, f"Field length must be at least {self.__threshold} characters"

        if isinstance(value, list):
            return len(value) >= self.__threshold, f"Field must have at least {self.__threshold} items"

        if isinstance(value, dict):
            return len(value.keys()) >= self.__threshold, f"Field must have at least {self.__threshold} items"

        return False, None

    def process(self, field: Any, value: Any) -> Any:
        return value


import abc
from typing import Any, Union, List

from fastvalidate.rule_abstract import RuleAbstract


class ComparisonRule(RuleAbstract, abc.ABC):

    def __init__(self, threshold: int) -> None:
        super().__init__()
        self._threshold = int(threshold)

    @abc.abstractmethod
    def compare(self, value: int, threshold: int) -> bool:
        pass

    def validate(self, field: Any, value: Any) -> bool:
        if isinstance(value, int): return self.compare(value, self._threshold)
        if isinstance(value, str) and value.isnumeric(): return self.compare(int(value), self._threshold)
        return False

    def process(self, field: Any, value: Any) -> Any:
        return int(value)


class GreaterThanRule(ComparisonRule):
    def compare(self, value: int, threshold: int) -> bool:
        return value > threshold

    @classmethod
    def slug(cls) -> Union[str, List[str]]:
        return ['gt']

    def message(self) -> str:
        return f'Field must be greater than {self._threshold}'


class GreaterThanEqualRule(ComparisonRule):
    def compare(self, value: int, threshold: int) -> bool:
        return value >= threshold

    @classmethod
    def slug(cls) -> Union[str, List[str]]:
        return ['gte']

    def message(self) -> str:
        return f'Field must be greater than or equal {self._threshold}'


class LessThanRule(ComparisonRule):
    def compare(self, value: int, threshold: int) -> bool:
        return value < threshold

    @classmethod
    def slug(cls) -> Union[str, List[str]]:
        return ['lt']

    def message(self) -> str:
        return f'Field must be less than {self._threshold}'


class LessThanEqualRule(ComparisonRule):
    def compare(self, value: int, threshold: int) -> bool:
        return value <= threshold

    @classmethod
    def slug(cls) -> Union[str, List[str]]:
        return ['lte']

    def message(self) -> str:
        return f'Field must be less than or equal {self._threshold}'



import re
from typing import Any, Union, List

from fastvalidate.rule_abstract import RuleAbstract


class EmailRule(RuleAbstract):

    @classmethod
    def slug(cls) -> Union[str, List[str]]:
        return ['email']

    def validate(self, field: Any, value: Any) -> bool:
        return isinstance(value, str) and re.match(r"[^@]+@[^@]+\.[^@]+", value)

    def process(self, field: Any, value: Any) -> Any:
        return value

    def message(self) -> str:
        return 'Field must be email'



from typing import Any, Union, List

from fastvalidate.rule_abstract import RuleAbstract


class BoolRule(RuleAbstract):

    @classmethod
    def slug(cls) -> Union[str, List[str]]:
        return ['boolean', 'bool']

    def validate(self, field: Any, value: Any) -> bool:
        return isinstance(value, bool) or value in ['true', 'false']

    def process(self, field: Any, value: Any) -> Any:
        if isinstance(value, bool): return value
        if value == 'true': return True
        return False

    def message(self) -> str:
        return 'Field must be boolean'



from typing import Any, Union, List

from fastvalidate.rule_abstract import RuleAbstract


class ChoiceRule(RuleAbstract):

    def __init__(self, *args) -> None:
        super().__init__()
        self.__options = args

    @classmethod
    def slug(cls) -> Union[str, List[str]]:
        return ['in', 'choice']

    def message(self) -> str:
        return f"Field must be one of {', '.join(self.__options)}"

    def validate(self, field: Any, value: Any) -> bool:
        return value in self.__options

    def process(self, field: Any, value: Any) -> Any:
        return value



import abc
from typing import Any, Union, List, Tuple, Optional


class RuleAbstract(abc.ABC):
    @classmethod
    @abc.abstractmethod
    def slug(cls) -> Union[str, List[str]]: pass

    @abc.abstractmethod
    def message(self) -> str: pass

    @abc.abstractmethod
    def validate(self, field: Any, value: Any) -> bool: pass

    @abc.abstractmethod
    def process(self, field: Any, value: Any) -> Any: pass


class ComplexRuleAbstract(RuleAbstract, abc.ABC):
    def __init__(self) -> None:
        super().__init__()
        self.__message: str = ''

    @abc.abstractmethod
    def validate_with_message(self, field: Any, value: Any) -> Tuple[bool, Optional[str]]:
        pass

    def validate(self, field: Any, value: Any) -> bool:
        result, message = self.validate_with_message(field, value)
        self.__message = message
        return result

    def message(self) -> str:
        return self.__message



import setuptools
from pathlib import Path

long_description = (Path(__file__).parent / "README.md").read_text()

setuptools.setup(
    name="fastvalidate",
    version="0.0.4",
    url="https://github.com/mojtabaahn/fast-validate",
    author="Mojtabaa Habibain",
    author_email="mojtabaa.hn@gmail.com",
    description="Easy validation for python",
    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
)